package XXV::MODULES::CHANNELS;

use strict;

use Tools;
use Event::File;
use Locale::gettext;
use File::stat;

# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'CHANNELS',
        Description => gettext('This module want read new Channels Data and save this in the Database.'),
        Version => '0.01',
        Date => '18.08.2004',
        Author => 'xpix',
        Preferences => {
            file => {
                description => gettext('Locate for channels.conf on youre system'),
                default     => '/var/lib/vdr/channels.conf',
                type        => 'file',
                required    => gettext('This is required!'),
            },
            interval => {
                description => gettext('The interval to parse channels to DB in sec'),
                default     => 300,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
            empty => {
                description => gettext('Include channels with empty PID'),
                default     => 'n',
                type        => 'confirm',
            },
            filterCA => {
                description => gettext('Filter channels, set all wanted CA(Common Access)'),
                # 0 for FTA, 1-4 for DVB Device, 32001 for AnalogPlugin
                default     => '',
                type        => 'string',
            },
            stripCH => {
                description => gettext("clean channelsnames, only part 'long' are visible"),
    			# Format in vdr 1.2.6 (Format "" or "long"). it show also all parts
    			# Format in vdr 1.3.10 (Format "short,long")
    			# Format in vdr 1.3.12 (Format "short,long;provider")
    			# Format in vdr 1.3.?? (Format "provider;short,long")
    			# Format in vdr 1.3.18 (Format "short,long;provider")
                default     => 'short,long;provider',
                type        => 'string',
            },

        },
        Commands => {
            cstatus => {
                description => gettext('Status from Channels Data'),
                short       => 'ct',
                callback    => sub{ $obj->status(@_) },
            },
            cupdate => {
                description => gettext('Read Channels and write into Database'),
                short       => 'cu',
                callback    => sub{ $obj->readData(@_) },
            },
            clist => {
                description => gettext("List Channels from database 'cname'"),
                short       => 'cl',
                callback    => sub{ $obj->list(@_) },
            },
        },
    };
    return $args;
}

# ------------------
sub status {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');

    my $sql = "select count(*) from CHANNELS";
    my $erg = $obj->{dbh}->selectall_arrayref($sql);

    $console->message(sprintf(gettext('%d Channels exists.'), $erg->[0][0]));
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # The Initprocess
    my $erg = $self->init or return error('Problem to initialze Module');

    # Interval to read channels and put to DB
    Event->timer(
        interval => $self->{interval},
        cb => sub{
            $self->readData();
        },
    );

	return $self;
}

# ------------------
sub init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # Old Version
    if(tableExists($obj->{dbh}, 'CHANNELS')) {
        $obj->{dbh}->do('drop table CHANNELS')
             unless(grep(/CA/,@{fields($obj->{dbh}, 'select * from CHANNELS')}));
    }

    # Look for table or create this table
    $obj->{dbh}->do(qq|
      CREATE TABLE IF NOT EXISTS CHANNELS (
          Id varchar(100) NOT NULL,
          Name varchar(100) NOT NULL default '',
          Frequency int(11) NOT NULL default '0',
          Parameters varchar(100) default '',
          Source varchar(100),
          Srate varchar(100) default '',
          VPID varchar(100) default '',
          APID varchar(100) default '',
          TPID varchar(100) default '',
          CA varchar(100) default '',
          SID varchar(100) default '',
          NID varchar(100) default '',
          TID varchar(100) default '',
          RID varchar(100) default '',
          POS int(11) NOT NULL,
          PRIMARY KEY  (Id)
        )
    |);

    my $erg = $obj->readData();

    return $erg;
}

# ------------------
sub insert {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $data = shift || return;
    my $pos = shift || return;

    return if($obj->{empty} eq 'n' and not (($data->[6] ne "0" || $data->[7] ne "0"))); # Ignore Channels with APID = 0/TPID = 0 from PID Scan
    if($obj->{filterCA}) {
       my $filter = $obj->{filterCA};
       $filter =~ s/\,/|/g; # Transform 0,2,400 => 0|2|400
       return 0 if(not ($data->[8] =~ /(^|\,)($filter)(\,|$)/s)); # check (^|,)(0|2|400)(,|$)
    }

	# Strip short and providername from channelname e.g ch
    if($obj->{stripCH}) {
		my $ch = $data->[0];
		my $filter = $obj->{stripCH};
		my @p = split(';',$filter);
		if(scalar @p == 2) {
			if(($p[0] =~ /provider/i)) { # format "provider;name"
				$ch = (split(';', $ch))[-1] if($ch =~ /;/);
				$filter = $p[1];
			}
			elsif(($p[1] =~ /provider/i)) { # format "name;provider"
				$ch = (split(';', $ch))[0] if($ch =~ /;/);
				$filter = $p[0];
			}
		}
		my @c = split(',',$filter);
		if(scalar @c == 2) {
			if(($c[0] =~ /long/i)) { # format "long,short"
				$ch = (split(',', $ch))[0] if($ch =~ /,/);
			}
			elsif(($c[1] =~ /long/i)) { # format "short,long"
				$ch = (split(',', $ch))[-1] if($ch =~ /,/);
			}
		}
	    $data->[0] = $ch if($ch);
	}

    # ID
    if ( $data->[3] eq 'C' or $data->[3] eq 'T') {
	    $data->[1]=substr($data->[1], 0, 3);
    }

    my $id;
    if($data->[12] && $data->[12] > 0 && $data->[12] < 100) {
        $id = sprintf('%s-%d-%d-%d-%d', $data->[3], $data->[10], ($data->[10] || $data->[11]) ? $data->[11] : $data->[1], $data->[9],$data->[12]);
    } else {
        $id = sprintf('%s-%d-%d-%d', $data->[3], $data->[10], ($data->[10] || $data->[11]) ? $data->[11] : $data->[1], $data->[9]);
    }
    unshift(@$data, $id);

    # POS
    push(@$data, $pos);

    my $sth = $obj->{dbh}->prepare('REPLACE INTO CHANNELS VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)');
    $sth->execute( @$data );
    return 1;
}

# ------------------
sub readData {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift;
    my $console = shift;
    my $file = $obj->{file} || return 1, warn ('No Channels File');

    return 1, warn ('No Channels File found') if( ! -e $file);

    # only if file modification from last read time
    my $mtime = (stat($file)->mtime);
    return
      if(defined $obj->{LastRefreshTime} and ($mtime eq $obj->{LastRefreshTime}));

    $obj->{dbh}->do('DELETE FROM CHANNELS');

    my $fh = IO::File->new("< $file") or return error("Can't open File $file $! ");
    my $c = 0;
    my $nPos = 1;
    while ( defined (my $line = <$fh>) ) {
        $line =~ s/[\r|\n]//sig;
        my @data = split(':', $line, 13);
        $c++
            if(scalar @data > 4 && $obj->insert(\@data, $nPos++));
    }
    $fh->close;

    $console->message(sprintf(gettext("Write %d Channels in Database."), $c))
        if(ref $console);
    $obj->{LastRefreshTime} = $mtime;
    return 1;
}

# ------------------
sub list {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $id      = shift || '';

    my $sql = "select * from CHANNELS where Name like '%$id%' order by POS";
    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);
    $console->table($erg);
}


# ------------------
sub NameToChannel {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $name = shift || return undef;

    my $sql = sprintf('select Id from CHANNELS where UPPER(Name) = UPPER("%s")', $name);
    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    return $erg->[0][0];
}

# ------------------
sub PosToName {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $pos = shift || return undef;

    my $sql = sprintf('select Name from CHANNELS where POS = "%d"', $pos);
    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    return $erg->[0][0];
}

# ------------------
sub PosToChannel {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $pos = shift || return undef;

    my $sql = sprintf('select Id from CHANNELS where POS = "%d"', $pos);
    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    return $erg->[0][0];
}

# ------------------
sub ChannelArray {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $field = shift || return undef;

    my $sql = sprintf('select %s, POS from CHANNELS order by POS', $field);
    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    return $erg;
}

# ------------------
sub ChannelHash {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $field = shift || return undef;

    my $sql = sprintf('select * from CHANNELS');
    my $erg = $obj->{dbh}->selectall_hashref($sql, $field);
    return $erg;
}

# ------------------
sub ChannelToName {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $id = shift || return undef;
    my $sql = sprintf('select Name from CHANNELS where Id = "%s"', $id);
    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    return $erg->[0][0];
}


1;
