package XXV::MODULES::HTTPD;

use Locale::gettext;
use XXV::OUTPUT::Html;
use IO::Socket::INET;
use MIME::Base64;
use CGI qw/-no_xhtml -compile/;
use File::Basename;
use File::Find;
use Tools;

use strict;

my $mime = {
    png  => "image/png",
    gif  => "image/gif",
    jpg  => "image/jpeg",
    css  => "text/css",
    ico  => "image/x-icon",
    js   => "application/x-javascript",
};

# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'HTTPD',
        Description => gettext('This module is a multisession HTTPD server.'),
        Version => '0.02',
        Date => '18.08.2004',
        Author => 'xpix',
        Preferences => {
            Clients => {
                description => gettext('Max allowed Connection in the same time'),
                default     => 5,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
            Port => {
                description => gettext('Portnumber to connect the client'),
                default     => 8080,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
            HtmlRoot => {
                description => gettext('tmpl files and images for http server (useful for skins)'),
                default     => 'html',
                type        => 'list',
                required    => gettext('This is required!'),
                choices     => $obj->findskins,
            },
        },
    };
    return $args;
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # The Initprocess
    $self->init or return error('Problem to initialze Module');

	return $self;
}

# ------------------
sub init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # globals
    my $channels;

    # make socket
	my $socket = IO::Socket::INET->new(
		Listen		=> $obj->{Clients} || 5,
		LocalPort	=> $obj->{Port}    || 8005,
		Reuse		=> 1);
	warn sprintf("[Fatal] Cannot build initial socket for %s on port %d. [%s]\n",
	        $obj->{Name}, $obj->{Port}, $!) unless $socket;

    # install an initial watcher
    Event->io(
        fd => $socket,
        cb => sub {
            # accept client
            my $client=$socket->accept;
            warn "[Error] Cannot connect http to new client.\n" and return unless $client;
            $client->autoflush;

            # make "channel" number
            my $channel=++$channels;

            # install a communicator
            Event->io(
                fd => $client,
                poll => 'r',
                cb => sub {
                    my $watcher = shift;

                    # read new line and report it
                    my $handle=$watcher->w->fd;

                    my $data = $obj->parseRequest($handle);
                    unless($data) {
                        $watcher->w->cancel;
                        $handle->close();
                        undef $watcher;
                        return 1;
                    }
                    my $htmlRootDir = sprintf('%s%s', $obj->{paths}->{HTMLDIR}, $obj->{HtmlRoot});
                    my $cgi = CGI->new( $data->{Query} );

                    my $console = XXV::OUTPUT::Html->new(
                        -handle => $handle,
                        -dbh    => $obj->{dbh},
                        -htmdir => $htmlRootDir,
                        -cgi    => $cgi,
                        -mime   => $mime,
                        -browser=> $data,
                        -paths  => $obj->{paths},
                    );

                    my $userMod = main::getModule('USER');
                    if($userMod->{Active} eq 'y') {
                        $obj->{USER} = $userMod->check($data->{username}, $data->{password});
                        $console->login(gettext('You have no permissions to this system!'))
                            unless(exists $obj->{USER}->{Name});
                    }

                    $console->{call} = 'nothing';
                    if(($data->{Request} eq '/' or $data->{Request} =~ /\.html$/) and not $data->{Query}) {
                        # Send the first page (index.html)
                        my $page = ($data->{Request} eq '/') ? 'index.html' : $data->{Request};
                        $console->datei(sprintf('%s/%s', $htmlRootDir, $page));
                    } elsif(my $typ = $mime->{lc((split('\.', $data->{Request}))[-1])}) {
                        # Send multimedia files (this must registered in $mime!)
                        $console->image(sprintf('%s%s', $htmlRootDir, $data->{Request}), $typ);
                    } elsif( $cgi->param('binary') ) {
                        # Send multimedia files (if param binary)
                        $obj->handleInput($watcher, $console, $cgi);
                    } else {
                        $console->start();
                        $obj->handleInput($watcher, $console, $cgi);
                        $console->footer();
                    }
                    $watcher->w->cancel;
                    undef $watcher;
                    $handle->close();
                },
            );
        },
    );
}

# ------------------
sub parseRequest {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $hdl = shift || return error ('No Handle!' );

    my $Req = getFromSocket($hdl);

	if($Req->[0] =~ /^GET (\/[\w\.\/-\:]*)([\?[\w=&\.\+\%-\:\!]*]*)[\#\d ]+HTTP\/1.\d$/) {
        my $data = {};
		($data->{Request}, $data->{Query}) = ($1, $2 ? substr($2, 1, length($2)) : undef);

    	# parse header
    	foreach my $line (@$Req) {
    		if($line =~ /Referer: (.*)/) {
    			$data->{Referer} = $1;
    		}
    		if($line =~ /Host: (.*)/) {
    			$data->{HOST} = $1;
    		}
    		if($line =~ /Authorization: basic (.*)/i) {
    			($data->{username}, $data->{password}) = split(":", MIME::Base64::decode_base64($1), 2);
    		}
    		if($line =~ /User-Agent: (.*)/i) {
    			$data->{http_useragent} = $1;
    		}
    	}
        return $data;
	} else {
           warn "Unknown Request: <%s>\n", join("\n", @$Req);
	   return;
	}

}

# ------------------
sub handleInput {
# ------------------
    my $obj     = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $cgi     = shift || return error ('No CGI Object');

    my $ucmd = $cgi->param('cmd')   || '<undef>';
    my $udata = $cgi->param('data') || '';

    # Test on result set (user has save) and
    # get the DataVars in a special Hash
    my $result;
    foreach my $name ($cgi->param) {
        if(my ($n) = $name =~ /^__(.+)/sig) {
            my @vals = $cgi->param($name);
            if(scalar @vals > 1) {
                @{$result->{$n}} = @vals;
            } else {
                $result->{$n} = shift @vals;
            }
        }
    }

    my $mods = main::getModules();
    # Search for command and call the callback
    my $ok = 0;
    foreach my $modName (keys %{$mods}) {
        my $modCfg = $mods->{$modName}->{MOD};
        foreach my $cmdName (sort keys %{$modCfg->{Commands}}) {
            if(lc($ucmd) eq $cmdName or lc($ucmd) eq $modCfg->{Commands}->{$cmdName}->{short}) {
                $ok++;
                $console->{call} = $cmdName;

                # Check Userlevel and Permissions
                my $u = main::getModule('USER');
                if($u->allowCommand($modCfg, $cmdName, $obj->{USER})) {
                    $modCfg->{Commands}->{$cmdName}->{callback}($watcher, $console, $udata, $result );
                } else {
                    return $console->message(gettext('You have no permissions on this command!'));
                }
            }
        }
    }

    unless($ok) {
        $obj->usage($watcher, $console, undef, sprintf(gettext("I don't understand the command '%s' \n"),$ucmd));
    }
}

# ------------------
sub usage {
# ------------------
    my $obj = shift || return error ('No Object!' );
    return main::getModule('TELNET')->usage(@_);
}

# ------------------
sub findskins
# ------------------
{
    my $obj = shift || return error ('No Object!' );
    my $found;
    find({ wanted => sub{
                if(-d $File::Find::name and -e $File::Find::name.'/index.html' ) {
                    my $l = basename($File::Find::name);
                    push(@{$found},[$l,$l]);
                }
           },
           follow => 1,
           follow_skip => 2,
        },
        $obj->{paths}->{HTMLDIR}
    );
    return $found;
}


1;
