package XXV::MODULES::RECORDS;

use strict;

use Tools;
use Time::Local;
use File::Find;
use File::Copy;
use File::Path;
use File::Basename;
use File::stat;
use Locale::gettext;

$SIG{CHLD} = 'IGNORE';

# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );

    my $args = {
        Name => 'RECORDS',
        Description => gettext('This module managed records.'),
        Version => '0.01',
        Date => '29.10.2004',
        Author => 'xpix',
        Preferences => {
            commandfile => {
                description => gettext('The place from the reccmds.conf file'),
                default     => '/var/lib/vdr/reccmds.conf',
                type        => 'file',
                required    => gettext("This is required!"),
            },
            interval => {
                description => gettext('The interval to read the records from svdrp'),
                default     => 300,
                type        => 'integer',
                required    => gettext("This is required!"),
            },
            videodir => {
                description => gettext('The dir where vdr save videofiles'),
                default     => '/video',
                type        => 'dir',
                required    => gettext("This is required!"),
            },
            mplayer => {
                description => gettext('Program to produce preview images from records'),
                default     => '/usr/bin/mplayer',
                type        => 'file',
                required    => gettext("This is required!"),
            },
            mplayercommand => {
                description => gettext('Please choose the version from you mplayer'),
                type        => 'list',
                choices     => [
                    ['No MPlayer'    , 'No MPlayer'],
                    ['MPlayer1.0pre5', 'MPlayer1.0pre5'],
                    ['MPlayer1.0pre6', 'MPlayer1.0pre6']
                ],
                default     => 'old',
                required    => gettext("This is required!"),
            },
            mplayerlog => {
                description => gettext('Logfile for mplayer'),
                default     => '/var/log/xxvd_mplayer.log',
                type        => 'string',
                required    => gettext("This is required!"),
            },
            vfat => {
                description => gettext('Set this if your filename encoded for vfat filesystems'),
                default     => 'y',
                type        => 'confirm',
            },
        },
        Commands => {
            rdisplay => {
                description => gettext("Display record 'rid'"),
                short       => 'rd',
                callback    => sub{ $obj->display(@_) },
            },
            rlist => {
                description => gettext('List records'),
                short       => 'rl',
                callback    => sub{ $obj->list(@_) },
            },
            rsearch => {
                description => gettext("Search records 'text'"),
                short       => 'rs',
                callback    => sub{ $obj->search(@_) },
            },
            rupdate => {
                description => gettext('Update record list'),
                short       => 'ru',
                callback    => sub{ $obj->readData(@_) },
                Level       => 'admin',
            },
            rdelete => {
                description => gettext("Delete record 'rid'"),
                short       => 'rr',
                callback    => sub{ $obj->delete(@_) },
                Level       => 'admin',
            },
            rrename => {
                description => gettext("Rename record 'rid' 'NewName'"),
                short       => 'rn',
                callback    => sub{ $obj->rrename(@_) },
                Level       => 'admin',
            },
            rconvert => {
                description => gettext("Convert record 'rid'"),
                short       => 'rc',
                callback    => sub{ $obj->convert(@_) },
                Level       => 'admin',
            },
        },
    };
    return $args;
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # i need the svdrp module
    $self->{svdrp} = main::getModule('SVDRP');

    # The Initprocess
    my $erg = $self->init or return error('Problem to initialze Module');

    # Interval to read channels and put to DB
    Event->timer(
        interval => $self->{interval},
        cb => sub{
            $self->readData();
        },
    );

	return $self;
}

# ------------------
sub init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # Old Version
    if(tableExists($obj->{dbh}, 'RECORDS')) {
        $obj->{dbh}->do('drop table RECORDS')
             unless(grep(/Path/,@{fields($obj->{dbh}, 'select * from RECORDS')}));
    }

    # Look for table or create this table
    $obj->{dbh}->do(qq|
      CREATE TABLE IF NOT EXISTS RECORDS (
          EventId int(100) NOT NULL,
          RecordId int(100) not NULL,
          Duration int(11) NOT NULL default '0',
          Start int(11) NOT NULL default '0',
          Path text NOT NULL,
          addtime timestamp,
          PRIMARY KEY  (EventId),
          UNIQUE KEY  (EventId)
        )
    |);

    # Load Reccmds's
    $obj->{reccmds} = [];
    if(-e $obj->{commandfile} and my $text = load_file($obj->{commandfile})) {
        foreach my $zeile (split(/\n/, $text)) {
            if($zeile !~ /^\#/ and $zeile !~ /^$/ and $zeile !~ /true/) {
                push(@{$obj->{reccmds}}, $zeile);
            }
        }
    }
    1;
}

# ------------------
sub readData {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift;
    my $console = shift;
    my $c = 0;
    $obj->{JOBS} = [];


    # Read VideoDir
    $obj->{FILES} = [];
    find(
            {
                wanted => sub{
                    push(@{$obj->{FILES}},$File::Find::name)
                        if($File::Find::name =~ /\d{3}.vdr$/sig);
                },
                follow => 1,
            },
        $obj->{videodir}
    );

    # Delete old Records
    $obj->{dbh}->do('DELETE FROM RECORDS');

    # Records einlesen
    my $erg = $obj->{svdrp}->command('lstr');

    my @merkIds;
    foreach my $record (@$erg) {
        if(my $anahash = $obj->analyze($record)) {
            push(@merkIds, $anahash->{EventId});
            $obj->insert($anahash);
            $c++;
        }
    }

    # Previews im fork erzeugen
    if(scalar @{$obj->{JOBS}}) {
        defined(my $child = fork()) or return error("Can't fork: $!");
        if($child == 0) {
            $obj->{dbh}->{InactiveDestroy} = 1;

            foreach my $command (@{$obj->{JOBS}}) {
                my $erg = system("nice -19 $command");
            }
            exit 0;
        }
    }

    # alte PreviewDirs loeschen
    foreach my $dir (glob(sprintf('%s%s/Preview/*', $obj->{paths}->{HTMLDIR}, main::getModule('HTTPD')->{HtmlRoot}))) {
        my $oldEventNumber = (split('/', $dir))[-1];
        unless(grep($_ == $oldEventNumber, @merkIds)) {
            $obj->deleteDir($dir);
        }
    }

    # Delete all EpgOld Entrys without the RecordIds
    if(scalar @merkIds) {
        my $sql = sprintf('delete from OLDEPG where eventid not in (%s)', join(',', @merkIds));
        $obj->{dbh}->do($sql);
    }

    if(ref $console) {
        $console->message(sprintf(gettext("Write %d Records in Database.\n"), $c));
        $console->link({
            text => gettext("Back to records list"),
            url => "?cmd=rlist",
        }) if($console->typ eq 'HTML');
    }
    return 1;
}

# ------------------
sub deleteDir {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $dir = shift || return;

    foreach my $file (glob(sprintf('%s/*.jpg', $dir))) {
        unlink $file;
    }
    rmdir $dir;
    return 1;
}

# ------------------
sub insert {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $attr = shift || return;

    my $sth = $obj->{dbh}->prepare('REPLACE INTO RECORDS(EventId, RecordId, Duration, Start, Path) VALUES (?,?,?,?,?)');
    $sth->execute(
        $attr->{EventId},
        $attr->{RecordId},
        $attr->{Duration},
        $attr->{Start},
        $attr->{Path},
    );
    return $obj->{dbh}->{'mysql_insertid'};
}

# ------------------
sub analyze {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $record = shift || return;
    my ($id, $day, $month, $houre, $minute, $state, $duration, $title);

    # New Format
    if($record =~ /\s+\d+\s+/) { # VDR is patched with recording length patch
        # 250-1 01.11 15:14* 50 Discovery~Die Rose von Kerrymore Spielfilm D/2000
        ($id, $day, $month, $houre, $minute, $state, $duration, $title)
            = $record =~ /^250[\-|\s](\d+)\s+(\d+)\.(\d+)\s+(\d+)\:(\d+)(.?)\s*(\d*).*?(\w.+)/si;
    } else { # Vanilla VDR
        # 250-1 01.11 15:14*  Discovery~Die Rose von Kerrymore Spielfilm D/2000
        ($id, $day, $month, $houre, $minute, $state, $title)
            = $record =~ /^250[\-|\s](\d+)\s+(\d+)\.(\d+)\s+(\d+)\:(\d+)(.?).*?(\w.+)/si;
    }
    return unless($id);

    my $info = $obj->videoInfo($day, $month, $houre, $minute, $title)
        or return error("Can not find record for Id $id!");

    return unless(ref $info eq 'HASH');

    my @t = split('~', $title);
    my $subtitle;
    if(scalar @t > 1) { # Spitt genre~title | subtitle
        $subtitle = delete $t[-1];
        $title = join('~',@t);
    }

    # XXX: Noch werden nicht alle Records korrekt erkannt
    my $year = ($info->{mtime}->[5] || 105) + 1900;
    my $start = timelocal(0,$minute,$houre,$day,$month-1, $year);
    my $dur = ($duration) ? $duration : (exists $info->{duration}) ? $info->{duration} : 0;
    my $event = $obj->SearchEpgId( $start, $dur, $title, $subtitle );

    # XXXX
    # Sollte kein Event gefunden werden so muss dieser in die EPGOLD mit
    # den vorhandenen Daten (lstr nummer) eingetragen werden und eine PseudoEventId (min(eventid)-1)
    # erfunden werden ;)
    unless($event) {
        $event = $obj->createOldEventId($id, $start, $duration, $title, $subtitle)
            || return error('Can not create OldEvent!');
    }

    # Make Preview
    my $command = $obj->videoPreview( $event, $info );
    push(@{$obj->{JOBS}}, $command)
        if($command);

    # Path to Record
    my $Path;
    if(scalar $info->{files} > 0 and $info->{files}->[0] ne "" ) {
        $Path = $info->{files}->[0];
        $Path = dirname($Path)
            if(! -d $Path);
    } else {
      warn ("couldn't found path for recording : $title / $subtitle");
      $Path = '<undef>';
    }

    my $ret = {
        RecordId => $id,
        Duration => $dur,
        Title => $title,
        Subtitle => $subtitle,
        Start => $start,
        State => ($state eq ' ' ? 0 : 1),
        Path  => $Path,
        EventId => $event->{eventid},
        Channel => $event->{channel_id},
    };
    return $ret;
}

# ------------------
sub videoInfo {
# ------------------
    my $obj     = shift || return error ('No Object!' );
    my $day     = shift || return error ('No Day!' );
    my $month   = shift || return error ('No Month!' );
    my $houre   = shift || return error ('No Houre!' );
    my $minute  = shift || return error ('No minute!' );
    my $title   = shift || return error ('No title!' );

    my @files;
    # Teste beide Moeglichkeiten aus (vfat=$obj->{vfat} und das Gegenteil
    foreach my $vfat ( ($obj->{vfat}, ($obj->{vfat} == 'y' ? 'n' : 'y') ) ) {
        my $t = $obj->translate($title, $vfat);
        $t =~ s/([\/\)\(\-])/\./g;
        @files = grep(/$t.*?\d{4}\-$month\-$day\.$houre[\:|\.]$minute/, @{$obj->{FILES}});
        if(scalar @files) {
            last;
        } else {
            warn sprintf('Can\'t find record with title: %s', $t);
        }
    }

    return 0, warn sprintf('Can\'t find record with title: %s', $title)
         unless(scalar @files);

    my $status = {};
    foreach my $file (@files) {
    	#Splitt 2005-01-16.04:35.88.99.rec
    	my ($year, $month, $day, $hour, $minute, $prio, $lifetime)
             = (basename(dirname($file))) =~ /^(\d+)\-(\d+)\-(\d+)\.(\d+)[\:|\.](\d+)\.(\d+)\.(\d+)\.rec/si;
    	if($year && $month && $day && $hour && $minute && $year >= 1970 && $year < 2038 ) {
    		@{$status->{mtime}} = localtime(timelocal(0,int($minute),int($hour),$day,$month-1,$year-1900));
    	} else {
            my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
        	    $atime,$mtime,$ctime,$blksize,$blocks) = stat $file;
    		@{$status->{mtime}} = localtime $mtime;
    	}
    }

    # Dateigre von index.vdr fr Aufnahmedauer ermitteln
    if($files[0] && -e $files[0]) {
        my $index = sprintf("%s/index.vdr", dirname($files[0]));
        if(-e $index) {
            my $bytes = stat($index)->size;
            $status->{duration} = ($bytes / 8) / 25;
        } else {
            warn "Couldn't read:", $index;
        }
    }

    $status->{files} = \@files;
    return $status;
}

# ------------------
sub videoPreview {
# ------------------
    my $obj     = shift || return error ('No Object!' );
    my $event   = shift || return error ('No EventHash!');
    my $info    = shift || return error ('No InfoHash!');

    if ($obj->{mplayercommand} eq 'No MPlayer') {
        return ();
    }

    # Videodir
    my $vdir    = dirname($info->{files}->[0]) || return warn $info->{files}->[0];

    # Save dir
    my $outdir = sprintf('%s%s/Preview/%d', $obj->{paths}->{HTMLDIR}, main::getModule('HTTPD')->{HtmlRoot}, $event->{eventid});
    my @files = glob("$outdir/*.jpg");
    if(scalar @files > 3) {
        return 0;
    } else {
        mkdir($outdir);
    }

    # Mplayer
    my $mplayer = $obj->{mplayer};
    return error('I can not found a mplayer executable file!') unless(-e $mplayer);

    my $scalex = 189;
    my $scaley = int($scalex / 4 * 3);
    my $frames = 4;
    my $file = $info->{files}->[0];
    my $startseconds = main::getModule('TIMERS')->{prevminutes} * 60 / 5;
    my $stepseconds = 120/5;

    my $mversions = {
        'MPlayer1.0pre5' => '%s -nosound -vo jpeg -jpeg outdir=\'%s\' -ni -ss %d -sstep %d -vf scale=%d:%d -frames %d \'%s\' >> %s 2>&1',
        'MPlayer1.0pre6' => '%s -nosound -vo jpeg:outdir=\'%s\' -ni -ss %d -sstep %d -vf scale=%d:%d -frames %d \'%s\' >> %s 2>&1',
    };


    my $command = sprintf($mversions->{$obj->{mplayercommand}},
                            $mplayer, $outdir, $startseconds, $stepseconds, $scalex, $scaley, $frames, $file, $obj->{mplayerlog});

    return $command;
}



# ------------------
sub SearchEpgId {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $start = shift || return error ('No Start!' );
    my $dur = shift || return error ('No Duration!' );
    my $title = shift || return error ('No Title!' );
    my $subtitle = shift || "";

    my $bis = $start + $dur;
    $title =~ s/\'/\\\'/sg;
    $subtitle =~ s/\'/\\\'/sg if($subtitle);
    $title =~ s/%/\%/sg;
    $subtitle =~ s/%/\%/sg if($subtitle);

    # Search in DB
    foreach my $table (qw/OLDEPG EPG/) {
        my $sql = "SELECT * FROM $table WHERE UNIX_TIMESTAMP(starttime) BETWEEN '$start' and '$bis' AND title = '$title'";
	    $sql .= " AND subtitle = '$subtitle'" if($subtitle);
        my $erg = $obj->{dbh}->selectrow_hashref($sql);
        return $erg
    		if($erg->{eventid}
    		  and ( # check for equal subtitle
    			(not $subtitle and not $erg->{subtitle})
    			 or (($subtitle and $erg->{subtitle}) and ($subtitle eq $erg->{subtitle}))
    		      )
    		  );
    }
}

# ------------------
sub createOldEventId {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $id = shift || return error ('No Id!' );
    my $start = shift || return error ('No Starttime!' );
    my $duration = shift || 0;
    my $title = shift || return error ('No Title!' );
    my $subtitle = shift || "";

    my $attr = {
        title => $title,
        subtitle => $subtitle,
        description => '',
        channel => '<undef>',
        duration => $duration,
        starttime => $start,
    };

    # get more info about the record
    my $erg = $obj->{svdrp}->command("lstr $id");
    foreach my $row (@$erg) {
        my @data = split(/\s+/, $row);
        if(shift @data eq '250') {
            $attr->{description} = join(' ', @data);
        }
    }

    $attr->{eventid} = $obj->{dbh}->selectall_arrayref('select max(eventid)+1 from OLDEPG')->[0][0];
    $attr->{eventid} = 1000000000 if($attr->{eventid} < 1000000000 );

    # dumper($attr);

    my $sth = $obj->{dbh}->prepare('REPLACE INTO OLDEPG(eventid, title, subtitle, description, channel_id, duration, tableid, starttime, addtime) VALUES (?,?,?,?,?,?,?,FROM_UNIXTIME(?),FROM_UNIXTIME(?))');
    $sth->execute(
        $attr->{eventid},
        $attr->{title},
        $attr->{subtitle},
        $attr->{description},
        $attr->{channel},
        $attr->{duration},
        $attr->{tableid},
        $attr->{starttime},
        time,
    );

    return $attr
}

# ------------------
sub display {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $recordid = shift || return $console->message(gettext("No RecordID to display the Record! Please use rdisplay 'rid'"));

    my $sql = qq|
select
    RECORDS.RecordId,
    RECORDS.EventId,
    RECORDS.Duration,
    DATE_FORMAT(FROM_UNIXTIME(RECORDS.Start), '%d.%m.%Y') as Day,
    DATE_FORMAT(FROM_UNIXTIME(RECORDS.Start), '%H:%i') as StartTime,
    DATE_FORMAT(FROM_UNIXTIME(RECORDS.Start + RECORDS.Duration), '%H:%i') as StopTime,
    OLDEPG.title as Title,
    OLDEPG.subtitle as SubTitle,
    OLDEPG.description as Description
from
    RECORDS,OLDEPG
where
    RECORDS.EventId = OLDEPG.eventid
    and RecordId = '$recordid'
|;

    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectrow_hashref($sql);

    # look for pictures
    my $outdir = sprintf('%s%s/Preview/%d', $obj->{paths}->{HTMLDIR}, main::getModule('HTTPD')->{HtmlRoot}, $erg->{EventId});
    my $param = {};
    if(my @previews = glob("$outdir/*.jpg")) {
        map {
            $_ =~ s/.+?Preview/Preview/sig
        } @previews;
        # Erstes Bild war immer Mist ;)
        shift @previews;
        $param->{previews} = \@previews;
    }
    $param->{reccmds} = $obj->{reccmds};
    $console->table($erg, $param);
}


# ------------------
sub list {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');

    my $sql = q|
SELECT
    r.RecordId as Id,
    r.EventId as __EventId,
    e.title,
    e.subtitle,
    r.duration,
    from_unixtime(r.Start) as RecordStart
FROM
    RECORDS as r,
    OLDEPG as e
WHERE
    e.eventid = r.EventId
ORDER BY
    r.Start desc
|;

    my $fields = fields($obj->{dbh}, $sql);
    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);
    $console->table($erg);

    return 1;
}

# ------------------
sub search {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $text      = shift || return $obj->list($watcher,$console);

    my $search = buildsearch("e.title,e.subtitle,e.description",$text);
    my $sql = qq|
SELECT
    r.RecordId as Id,
    r.EventId as __EventId,
    e.title,
    e.subtitle,
    r.duration,
    from_unixtime(r.Start) as RecordStart
FROM
    RECORDS as r,
    OLDEPG as e
WHERE
    e.eventid = r.EventId
	AND ( $search )
ORDER BY
    r.Start desc
|;

    my $fields = fields($obj->{dbh}, $sql);
    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);
    $console->table($erg);

    return 1;
}

# ------------------
sub delete {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $record  = shift || return $console->message(gettext('No Record to delete!'));
    my $erg = $obj->{svdrp}->command("delr $record");

    $console->message(sprintf(gettext("Record with number '%s' to delete ..."),$record));
    sleep(2);
    $obj->readData($watcher, $console);
    return 1;
}


sub is_empty_dir {
    my $dir    = shift;
    local (*DIR);
    return unless opendir DIR, $dir;
    while (defined($_ = readdir DIR)) {
        next if /^\.\.?$/;
        closedir DIR;
        return 0;
    }
    closedir DIR;
    return 1;
}

sub touch {
    my $files    = shift;
    my $now = time;
    local (*TMP);
    foreach my $file ($files) {
            utime ($now, $now, $file)
            || open (TMP, ">>$file")
            || warn ("Couldn't touch $file: $!\n");
    }
}

# ------------------
sub rrename {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $recordid  = shift || return $console->message(gettext("No RecordID to rename!"));
    my $data    = shift || 0;

    my $rec;
    if($recordid) {
        my $sql = "select CONCAT_WS('~',OLDEPG.title,OLDEPG.subtitle) as OldName, RECORDS.Path as OldPath, RECORDS.EventId as EventId from RECORDS,OLDEPG where RECORDS.EventId = OLDEPG.eventid and RecordId = '$recordid'";

        $rec = $obj->{dbh}->selectrow_hashref($sql)
            or return $console->message(sprintf(gettext("RecordID '%s' is not exist in Database!"),$recordid));
    }

    my $questions = [
        'newname' => {
            msg   => gettext("New Name"),
            def   => $rec->{OldName} || '',
        },
    ];

    $data = $console->question(gettext("Rename Record"), $questions, $data);

    return 1 unless($data->{newname});

    if(ref $data eq 'HASH') {
        # Rename auf der Platte
        my ($old) = $rec->{OldPath} =~ /^(.+?)\/\d{4}\-\d{2}\-\d{2}.+?rec/;
        my $new = sprintf('%s/%s', $obj->{videodir}, $obj->translate($data->{newname}));

        my $parentnew = dirname($new);
        unless( -d $parentnew) {
            mkpath($parentnew) or return $console->message(sprintf(gettext("Record: '%s', can't mkdir: '%s' : %s"),$rec->{OldName},$parentnew,$!));
        }

        move($old,$new) or return $console->message(sprintf(gettext("Record: '%s', can't move to '%s' : %s"),$rec->{OldName},$data->{newname},$!));

        my $parentold = dirname($old);
        if($obj->{videodir} ne $parentold
            and -d $parentold
            and is_empty_dir($parentold)) {
            rmdir($parentold) or warn (sprintf("Record: '%s', can't remove '%s' : %s",$rec->{OldName},$parentold,$!));
        }
        #Ab 1.3.11 resync with touch /video/.update
        touch($obj->{videodir}."/.update");

        my $title = $data->{newname};
        $title =~ s/\'/\\\'/g; # Replace ' => \'

        my @t = split('~', $title); # Generate Subtitle
        if(scalar @t > 1) { # Spitt genre~title | subtitle
            my $subtitle = $t[-1];
            delete $t[-1];
            $title = join('~',@t);
            # TODO Merge SQL-Statement
            $obj->{dbh}->do(sprintf("update OLDEPG set title = '%s' where eventid = %d",$title, $rec->{EventId}));
            $obj->{dbh}->do(sprintf("update OLDEPG set subtitle = '%s' where eventid = %d",$subtitle, $rec->{EventId}));
        }
        else {
            # TODO Merge SQL-Statement, to one
            $obj->{dbh}->do(sprintf("update OLDEPG set title = '%s' where eventid = %d",$title, $rec->{EventId}));
            $obj->{dbh}->do(sprintf("update OLDEPG set subtitle = NULL where eventid = %d",$rec->{EventId}));
        }

        $console->message(sprintf(gettext("Record: '%s' renamed to '%s'"),$rec->{OldName}, $data->{newname} ));

        sleep(2);
        $obj->readData($watcher, $console);
    }

    return 1;
}

# ------------------
sub convert {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $data = shift || 0;


    unless(scalar @{$obj->{reccmds}}) {
        $console->message(gettext('No reccmds.conf on your System!'));
        return 1;
    }

    unless($data) {
        $console->message(gettext("Please use rconvert 'cmdid_rid'"));
        unshift(@{$obj->{reccmds}}, ['Descr.', 'Command']);
        $console->table($obj->{reccmds});
        $obj->list($watcher, $console);
    }

    my ($cmdid, $recid) = split(/[\s_]/, $data);
    my $cmd = (split(':', $obj->{reccmds}->[$cmdid-1]))[-1] || return $console->message(gettext("I can't find this CommandID"));
    my $path = $obj->IdToPath($recid) || return $console->message(gettext("I can't find this RecordID"));

    my $call = "$cmd \"$path\"";
    my $output = `$call`;
    if( $? >> 8 > 0) {
        $console->message(sprintf(gettext("Sorry! Call %s %s Error output: %s"), $cmd, $path, $output));
    } else {
        $console->message(sprintf(gettext("Call %s %s With output: %s"), $cmd, $path, $output));
    }
    $console->link({
        text => gettext("Back to records list"),
        url => "?cmd=rlist",
    }) if($console->typ eq 'HTML');
    return 1;
}

# ------------------
sub IdToPath {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $id = shift || return undef;

    my $sql = sprintf('select Path from RECORDS where RecordId = "%d"', $id);
    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    return $erg->[0][0];
}

# ------------------
sub translate {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $title = shift || return error ('No Title in translate!');
    my $vfat = shift || $obj->{vfat};

    if($vfat eq 'y')
    {
        $title =~ s/([^a-z0-9\&\!\-\s\.\@\~\,\(\)\%])/sprintf('#%X', ord($1))/seig;
        $title =~ s/[^a-z0-9\!\&\-\#\.\@\~\,\(\)\%]/_/sig;
    } else {
        $title =~ s/\'/\x01/g;
        $title =~ s/\\/\x02/g;
        $title =~ s/ /_/g;
    }
    $title =~ s/~/\//g;

    return $title;
}

1;
