package XXV::MODULES::TIMERS;

use strict;
use Tools;
use POSIX ":sys_wait_h", qw(strftime mktime);
use Event::File;
use Locale::gettext;

# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'TIMERS',
        Description => gettext('This module parse the timers.conf and save this in the database.'),
        Version => '0.01',
        Date => '06.09.2004',
        Author => 'xpix',
        Preferences => {
            file => {
                description => gettext('Locate for timers.conf on youre system'),
                default     => '/var/lib/vdr/timers.conf',
                type        => 'file',
                required    => gettext("This is required!"),
            },
            interval => {
                description => gettext('The interval to parse timers to database in seconds'),
                default     => 300,
                type        => 'integer',
                required    => gettext("This is required!"),
            },
            prevminutes => {
                description => gettext('The minutes, befor start the program for record'),
                default     => 5,
                type        => 'integer',
            },
            afterminutes => {
                description => gettext('The minutes, after end the recording program'),
                default     => 5,
                type        => 'integer',
            },
            Priority => {
                description => gettext('Defining the priority of this timer and of recordings created by this timer.'),
                default     => 50,
                type        => 'integer',
            },
            Lifetime => {
                description => gettext('The guaranteed lifetime (in days) of a recording created by this timer'),
                default     => 50,
                type        => 'integer',
            },
        },
        Commands => {
            tstatus => {
                description => gettext("Status from timers"),
                short       => 'ta',
                callback    => sub{ $obj->status(@_) },
            },
            tlist => {
                description => gettext("List timers 'tid'"),
                short       => 'tl',
                callback    => sub{ $obj->list(@_) },
            },
            tsearch => {
                description => gettext("Search timers 'text'"),
                short       => 'ts',
                callback    => sub{ $obj->search(@_) },
            },
            tupdate => {
                description => gettext("Read timers and write into database"),
                short       => 'tu',
                callback    => sub{ $obj->readData(@_) },
                Level       => 'user',
            },
            tnew => {
                description => gettext("Create timer 'eid'"),
                short       => 'tn',
                callback    => sub{ $obj->newTimer(@_) },
                Level       => 'user',
            },
            tedit => {
                description => gettext("Edit timer 'tid'"),
                short       => 'te',
                callback    => sub{ $obj->editTimer(@_) },
                Level       => 'user',
            },
            tdelete => {
                description => gettext("Delete timer 'tid'"),
                short       => 'td',
                callback    => sub{ $obj->deleteTimer(@_) },
                Level       => 'user',
            },
            ttoogle => {
                description => gettext("Activate/Deactive timer 'tid'"),
                short       => 'tt',
                callback    => sub{ $obj->toogleTimer(@_) },
                Level       => 'user',
            },
        },
    };
    return $args;
}

# ------------------
sub status {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');

    my $sql = "select count(*) from TIMERS";
    my $erg = $obj->{dbh}->selectall_arrayref($sql);

    $console->message(sprintf('%d Timers exists.', $erg->[0][0]));
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_} || $self->{MOD}->{Preferences}->{$_}->{default}
    } keys %{$self->{MOD}->{Preferences}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # The Initprocess
    my $erg = $self->init or return error('Problem to initialze Module');

    # Interval to read timers and put to DB
    Event->timer(
        interval => $self->{interval},
        cb => sub{
            $self->readData();
        },
    );

	return $self;
}

# ------------------
sub init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # Old Version
    if(tableExists($obj->{dbh}, 'TIMERS')) {
        $obj->{dbh}->do('drop table TIMERS')
            unless(grep(/Collision/, fields($obj->{dbh}, 'select * from TIMERS')));
    }

    # Look for table or create this table
    $obj->{dbh}->do(qq|
      CREATE TABLE IF NOT EXISTS TIMERS (
          Id int(11) NOT NULL,
          Status char(1) default 1,
          ChannelID varchar(100) NOT NULL default '',
          Day varchar(100) default '-------',
          Start int(11),
          Stop int(11),
          Priority int(11),
          Lifetime int(11),
          File varchar(100),
          Summary text default '',
          NextStartTime datetime,
          NextStopTime datetime,
          Collision varchar(100) default '0',
          PRIMARY KEY  (Id)
        )
    |);

    $obj->{LastRefreshTime} = time;

    my $erg = $obj->readData();

    return $erg;
}

# ------------------
sub search {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $text      = shift || return $obj->list($watcher,$console);

	my $search = buildsearch("t.File,t.Summary",$text);
    $obj->readData;

    my $sql = qq|
SELECT
    t.Id,
    t.Status,
    c.Name as Channel,
    c.Pos as __Pos,
    t.Day as Prg,
    t.Start,
    t.Stop,
    t.File,
    DATE_FORMAT(t.NextStartTime, '%a %d.%m') as Day,
    t.Collision as __Collision
FROM
    TIMERS as t,
    CHANNELS as c
WHERE
    t.ChannelID = c.Id
    and ( $search )
ORDER BY
    t.NextStartTime|;

    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);
    $console->table($erg, {runningTimer => $obj->getRunningTimer});
}

# ------------------
sub saveTimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $data = shift || return error('No Data to Save!');
    my $timerid = shift || 0;

    $data->{File} =~ s/:/|/g;

    my $erg = $obj->cmd(
        sprintf("%s %s:%s:%s:%s:%s:%s:%s:%s:%s",
            $timerid ? "modt $timerid" : "newt",
            ($data->{Status} eq 'y' ? 1 : 0),
            $data->{ChannelID},
            $data->{Day},
            $data->{Start},
            $data->{Stop},
            $data->{Priority},
            $data->{Lifetime},
            $data->{File},
            ($data->{Summary} || ''),
        )
    );

    # Save shortly this timer in DB if this only a new timer (at)
    if(! $timerid and my $pos = $erg->[1] =~ /250\s+(\d+)/) {
        $obj->insert([
                ($data->{Status} eq 'y' ? 1 : 0),
                $data->{ChannelID},
                $data->{Day},
                $data->{Start},
                $data->{Stop},
                $data->{Priority},
                $data->{Lifetime},
                $data->{File},
                ($data->{Summary} || ''),
                ], $pos);
    }
    return $erg;
}

# ------------------
sub newTimer {
# ------------------
    my $obj     = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $epgid   = shift || 0;
    my $epg     = shift || 0;


    if($epgid and not ref $epg) {
        $epg = $obj->{dbh}->selectrow_hashref(qq|
SELECT
    eventid,
    channel_id,
    description as Summary,
    CONCAT_WS('~', title, subtitle) as File,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(starttime) - ( $obj->{prevminutes} * 60 )), '%d') as Day,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(starttime) - ( $obj->{prevminutes} * 60 )), '%H%i') as Start,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(starttime) + duration + ( $obj->{afterminutes} * 60 )), '%H%i') as Stop
FROM
    EPG
WHERE
    eventid = $epgid
|
        ) or return $console->message(sprintf(gettext("Event ID '%s' is not exist in database!"),$epgid));
    }
    if(ref $epg) {
        $epg->{Status} = 'y';
    } else {
		my $t = time;
   	    $epg = {
            Status => 'y',
            channel_id => '',
            File => '',
            Summary => '',
            Day => my_strftime("%d",$t),
            Start => my_strftime("%H%M",$t),
            Stop => my_strftime("%H%M",$t)
    	};
    }

    $obj->editTimer($watcher, $console, 0, $epg);
}

# ------------------
sub editTimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $timerid = shift || 0;   # If timerid the edittimer
    my $data    = shift || 0;  # Data for defaults

    my $epg;
    if($timerid and not ref $data) {
        $epg = $obj->{dbh}->selectrow_hashref("select Id, ChannelID as channel_id, File, Summary, Start, Stop, Day, Priority, Lifetime, Status from TIMERS where Id = $timerid")
            or return $console->message(sprintf(gettext("Timer ID '%s' is not exist in database!"),$timerid));
    } elsif (ref $data eq 'HASH') {
        $epg = $data;
    }

    $epg->{Summary} =~ s/(\r|\n)//sig
        if(defined $epg->{Summary});

    my $mod = main::getModule('CHANNELS');

    my $questions = [
        'Id' => {
            typ     => 'hidden',
            def     => $epg->{Id} || 0,
        },
        'Status' => {
            typ     => 'confirm',
            def     => (defined $epg->{Status} and $epg->{Status} ? 'y' : 'n'),
            msg     => gettext('Switch this timer on?'),
        },
        'ChannelID' => {
            typ     => 'list',
            def     => $mod->ChannelToName($epg->{channel_id}),
            choices => $mod->ChannelArray('Name'),
            msg     => gettext('Which channel should recorded?'),
            req     => gettext("This is required!"),
            check   => sub{
                my $value = shift || return;

                if(my $ch = $mod->NameToChannel($value) || $mod->PosToChannel($value)) {
                    $epg->{Channel} = $value;
                    return $ch;
                } elsif( ! $mod->NameToChannel($value)) {
                    return undef, sprintf(gettext("This channel '%s' is not exists!"),$value);
                } else {
                   return undef, gettext("This is required!");
                }
            },
        },
        'Day' => {
            typ     => 'string',
            def     => $epg->{Day},
            msg     => gettext("Please enter a day (1 to 31) or the weekday in format 'MDMDFSS'."),
            req     => gettext("This is required!"),
            check   => sub{
                my $value = shift || return;
                if(($value =~ /^\d+$/ and int($value) <= 31 and int($value) > 0) or $value =~ /^\S{7}\@\d{4}\-\d{2}-\d{2}$/sig or $value =~ /^\S{7}$/) {
                    return $value;
                } else {
                    return undef, gettext('No right day or corrupt format!');
                }
            },
        },
        'Start' => {
            typ     => 'string',
            def     => sub{
                my $value = sprintf('%04d',$epg->{Start} || 0 );
                return substr($value, 0, 2) .':'. substr($value, 2, 2);
                },
            msg     => gettext("Starttime in format 'HH:MM'"),
            check   => sub{
                my $value = shift || return;
                my @v = split(':', $value);
                return undef, gettext('No right time!') if(scalar @v != 2);
                $value = sprintf('%02d%02d',$v[0],$v[1]);
                if(int($value) < 2400 and int($value) >= 0) {
                    return sprintf('%04d',$value);
                } else {
                    return undef, gettext('No right time!');
                }
            },
        },
        'Stop' => {
            typ     => 'string',
            def     => sub{
                my $value = sprintf('%04d',$epg->{Stop} || 0 );
                return substr($value, 0, 2) .':'. substr($value, 2, 2);
                },
            msg     => gettext("Endtime in format 'HH:MM'"),
            check   => sub{
                my $value = shift || return;
                my @v = split(':', $value);
                return undef, gettext('No right time!') if(scalar @v != 2);
                $value = sprintf('%02d%02d',$v[0],$v[1]);
                if(int($value) < 2400 and int($value) >= 0) {
                    return sprintf('%04d',$value);
                } else {
                    return undef, gettext('No right time!');
                }
            },
        },
        'Priority' => {
            typ     => 'integer',
            msg     => gettext('Priority (0 .. 99)'),
            def     => $epg->{Priority} || $obj->{Priority},
            check   => sub{
                my $value = shift || return;
                if($value > 0 and $value < 100) {
                    return int($value);
                } else {
                    return undef, gettext('No right Value!');
                }
            },
        },
        'Lifetime' => {
            typ     => 'integer',
            msg     => gettext('Lifetime (0 .. 99)'),
            def     => $epg->{Lifetime} || $obj->{Lifetime},
            check   => sub{
                my $value = shift || return;
                if($value > 0 and $value < 100) {
                    return int($value);
                } else {
                    return undef, gettext('No right Value!');
                }
            },
        },
        'File' => {
            msg     => gettext('Title of recording'),
            def     => $epg->{File},
            req     => gettext("This is required!"),
        },
        'Summary' => {
            msg   =>  gettext('Additional description'),
            def   => $epg->{Summary},
            check   => sub{
                my $value = shift || return;
                $value =~ s/(\r|\n)//sig;
                return $value;
            },
        },
    ];

    # Ask Questions
    my $datasave = $console->question(($timerid ? gettext('Edit timer')
                                                : gettext('New timer')), $questions, $data);

    if(ref $datasave eq 'HASH') {
        my $erg = $obj->saveTimer($datasave, $timerid);
        $obj->readData();

        $console->message($erg);
        $console->link({
            text => gettext("Back to timer list"),
            url => "?cmd=tlist",
        }) if($console->typ eq 'HTML');
    }
}

# ------------------
sub deleteTimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $timerid = shift || return $console->message(gettext("No Timer ID to delete! Please use tdelete 'tid'"));   # If timerid the edittimer
    my $answer  = shift || 0;

    my @timers  = reverse sort{ $a <=> $b } split(/[^0-9]/, $timerid);

    my $data = $obj->{dbh}->selectrow_hashref(sprintf("select * from TIMERS where Id in (%s)", join(',', @timers)))
        or $console->message(sprintf(gettext("Timer ID '%s' is not exist in database!"),$timerid));

    my $mod = main::getModule('CHANNELS');
    $data->{Channel} = $mod->ChannelToName($data->{ChannelID});

    if($console->{TYP} ne 'HTML') {
        $console->table($data);
        my $confirm = $console->question({
            typ   => 'confirm',
            def   => 'y',
            msg   => gettext('Are you sure to delete this timer?'),
        }, $answer);
        return 1 unless($answer or $confirm eq 'y');
    }

    for my $timer (@timers) {
        my $erg = $obj->cmd("delt $timer");
        $console->message($erg);
    }

    $console->link({
        text => gettext("Back to timer list"),
        url => "?cmd=tlist",
    }) if($console->typ eq 'HTML');

    sleep(1);

    $obj->readData();

    return 1;
}

# ------------------
sub toogleTimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $timerid = shift || return $console->message(gettext("No Timer ID to toogle! Please use ttoogle 'id'"));   # If timerid the edittimer

    my @timers  = reverse sort{ $a <=> $b } split(/[^0-9]/, $timerid);

    for my $timer (@timers) {
	my $data = $obj->{dbh}->selectrow_hashref(sprintf("select * from TIMERS where Id in (%s)", $timer))
	   or return $console->message(sprintf(gettext("Timer ID '%s' is not exist in database!"),$timer));
    	my $status = ($data->{Status} ? 'off' : 'on');
    	my $erg = $obj->cmd("modt $data->{Id} $status");
    	$console->message($erg);
    }

    $obj->readData();
}


# ------------------
sub insert {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $data = shift || return;
    my $pos = shift || return;

    # POS
    unshift(@$data, $pos);
    $data->[8] =~ s/\|/\:/g;

    # NextTime
    my $nexttime = $obj->getNextTime( $data->[3], $data->[4], $data->[5] )
        or return error(sprintf("Can not get the nextime for this data: %s", join(' ', @$data)));
    push(@$data, $nexttime->{start}, $nexttime->{stop});

    my $sth = $obj->{dbh}->prepare('REPLACE INTO TIMERS VALUES (?,?,?,?,?,?,?,?,?,?,FROM_UNIXTIME(?), FROM_UNIXTIME(?),0)');
    $sth->execute( @$data );
}


# Read from svdrp (better for future development)
# ------------------
sub readData {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift;
    my $console = shift;
    my $file = $obj->{file} || return 1, warn('No File');

    $obj->{dbh}->do('DELETE FROM TIMERS');

	my $fh = IO::File->new("< $file") or return 1, warn("Can't open file $file $! ");
    my $c = 0;
	while ( defined (my $line = <$fh>) ) {
        $c++;
        my @data = split(':', $line, 9);
        $obj->insert(\@data, $c)
            if(scalar @data > 2);

	}
	$fh->close;

    $obj->getOverlappingTimer();

    $console->message(sprintf(gettext("Write %d timers in database."), $c))
        if(ref $console);

    $obj->{LastRefreshTime} = time;

    return 1;
}

# ------------------
sub list {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $id      = shift || '';

    $obj->readData;

    my $sql = qq|
SELECT
    t.Id,
    t.Status,
    c.Name as Channel,
    c.Pos as __Pos,
    t.Day as Prg,
    t.Start,
    t.Stop,
    t.File,
    DATE_FORMAT(t.NextStartTime, '%a %d.%m') as Day,
    t.Collision as __Collision
FROM
    TIMERS as t,
    CHANNELS as c
WHERE
    t.ChannelID = c.Id
    and t.Id like '%$id%'
ORDER BY
    t.NextStartTime|;

    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);
    $console->table($erg, {runningTimer => $obj->getRunningTimer});
}

# ------------------
sub cmd {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $cmd = shift || return error ('No Command!' );
    $obj->{MODULES} = main::getModules()
        unless(ref $obj->{MODULES});

    my $ret = $obj->{MODULES}->{'XXV::MODULES::SVDRP'}->command($cmd);
    return $ret;
}

# ------------------
sub getRunningTimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $sql = "select * from TIMERS where NOW() between NextStartTime and NextStopTime AND Status > 0 ";
    my $erg = $obj->{dbh}->selectall_hashref($sql, 'Id');
    return $erg;
}

# ------------------
sub getOverlappingTimer {
# ------------------
    my $obj  = shift || return error ('No Object!' );

    return unless(tableExists($obj->{dbh}, 'CHANNELS'));

    my $sql = 'select TIMERS.Id, TIMERS.NextStartTime, TIMERS.NextStopTime, CHANNELS.TID as tid, LEFT(CHANNELS.Source,1) as source from TIMERS, CHANNELS where Status > 0 and TIMERS.ChannelID = CHANNELS.Id';
    my $erg = $obj->{dbh}->selectall_hashref($sql, 'Id');

    foreach my $tid (keys %$erg) {
        my $sql = sprintf(
qq|
select
    TIMERS.Id
from
    TIMERS, CHANNELS
where
    ('%s' between TIMERS.NextStartTime
        and TIMERS.NextStopTime
        OR '%s' between TIMERS.NextStartTime
        and TIMERS.NextStopTime)
    and
        TIMERS.Id != %d AND TIMERS.Status > 0
    and
        TIMERS.ChannelID = CHANNELS.Id
    and
        CHANNELS.TID != %d
    and
        LEFT(CHANNELS.Source,1) = '%s'
            |, $erg->{$tid}->{NextStartTime}, $erg->{$tid}->{NextStopTime}, $tid, $erg->{$tid}->{tid},$erg->{$tid}->{source});
        my $result = $obj->{dbh}->selectall_hashref($sql, 'Id');

        if(scalar keys %$result) {
            $obj->{dbh}->do(sprintf('UPDATE TIMERS SET Collision = "%s" WHERE Id = %d', join(',',keys %$result), $tid));
        }
    }
}

# The following subroutines is stolen from vdradmind
# Thanks on Cooper and Thomas for this great work!
# $obj->getNextTime('MDMDFSS', 1300, 1200)
# ------------------
sub getNextTime {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $dor = shift || return error ('No Day!' );
    my $start = shift || return error ('No Starttime!' );
    my $stop =  shift || return error ('No Stoptime!' );

    $start = sprintf('%04d', $start);
    $stop = sprintf('%04d', $stop);

    my ($startsse, $stopsse);
    if(length($dor) == 7) { # repeating timer
        $startsse = my_mktime(substr($start, 2, 2), substr($start, 0, 2),
                       my_strftime("%d"), (my_strftime("%m") - 1), my_strftime("%Y"));
        $stopsse  = my_mktime(substr($stop, 2, 2), substr($stop, 0, 2),
                       my_strftime("%d"), (my_strftime("%m") - 1), my_strftime("%Y"));
        $stopsse += 86400 if($stopsse < $startsse);

        my $weekday = ((localtime(time))[6] + 6) % 7;
        my $perrec = join("", substr($dor, $weekday), substr($dor, 0, $weekday));
        $perrec =~ m/^-+/g;

        my $off = (pos $perrec || 0) * 86400;
        if($off == 0 && $stopsse < time) {
            #$weekday = ($weekday + 1) % 7;
            $perrec = join("", substr($dor, ($weekday + 1) % 7), substr($dor, 0, ($weekday + 1) % 7));
            $perrec =~ m/^-+/g;
            $off = ((pos $perrec || 0) + 1) * 86400;
        }
        $startsse += $off;
        $stopsse += $off;
    } elsif(length($dor) == 18) { # first-day timer
        $dor =~ /.{7}\@(\d\d\d\d)-(\d\d)-(\d\d)/;
        $startsse = my_mktime(substr($start, 2, 2),
                substr($start, 0, 2), $3, ($2 - 1), $1);
        # 31 + 1 = ??
        $stopsse = my_mktime(substr($stop, 2, 2),
                substr($stop, 0, 2), $stop > $start ? $3 : $3 + 1,
                ($2 - 1), $1);
    } else { # regular timer
        $startsse = my_mktime(substr($start, 2, 2),
                substr($start, 0, 2), $dor, (my_strftime("%m") - 1),
                my_strftime("%Y"));

        $stopsse = my_mktime(substr($stop, 2, 2),
                substr($stop, 0, 2), $stop > $start ? $dor : $dor + 1,
                (my_strftime("%m") - 1), my_strftime("%Y"));
    }

    # move timers which have expired one month into the future
    if(length($dor) != 7 && $stopsse < time) {
      $startsse = my_mktime(substr($start, 2, 2),
      substr($start, 0, 2), $dor, (my_strftime("%m") % 12),
      (my_strftime("%Y") + (my_strftime("%m") == 12 ? 1 : 0)));

      $stopsse = my_mktime(substr($stop, 2, 2),
      substr($stop, 0, 2), $stop > $start ? $dor : $dor + 1,
      (my_strftime("%m") % 12),
      (my_strftime("%Y") + (my_strftime("%m") == 12 ? 1 : 0)));
    }

    my $ret = {
        start => $startsse,
        stop => $stopsse,
    };
    return $ret;
}

sub my_mktime {
    my $sec  = 0;
    my $min = shift;
    my $hour = shift;
    my $mday = shift;
    my $mon  = shift;
    my $year = shift() - 1900;

    return mktime($sec, $min, $hour, $mday, $mon, $year, 0, 0, -1);
}

sub my_strftime {
    my $format = shift;
    my $time = shift || time;
    return(strftime($format, localtime($time)));
}


1;
